import os
from google.cloud import storage
from pose_format import Pose
import tarfile

from tqdm import tqdm


def fix_pose_file(file_path: str):
    with open(file_path, "rb") as buffer:
        pose = Pose.read(buffer.read())
        if len(pose.header.components) == 4:
            return

    print(f'Fixing {file_path}')

    pose = pose.get_components(
        ["POSE_LANDMARKS", "FACE_LANDMARKS", "LEFT_HAND_LANDMARKS", "RIGHT_HAND_LANDMARKS"])

    with open(file_path, "wb") as f:
        pose.write(f)


def download_blob(bucket_name, source_blob_name, destination_file_name):
    """Downloads a blob from the bucket."""
    storage_client = storage.Client()
    bucket = storage_client.bucket(bucket_name)
    blob = bucket.blob(source_blob_name)

    blob.download_to_filename(destination_file_name)
    print(f'Blob {source_blob_name} downloaded to {destination_file_name}.')

    fix_pose_file(destination_file_name)


def list_blobs_with_prefix(bucket_name, prefix):
    """Lists all the blobs in the bucket with the given prefix."""
    storage_client = storage.Client()
    blobs = storage_client.list_blobs(bucket_name, prefix=prefix)

    return blobs


def validate_pose_file(file_name: str, buffer: bytes):
    pose = Pose.read(buffer)
    data_frames, data_people, data_points, data_dimensions = pose.body.data.shape
    conf_frames, conf_people, conf_points = pose.body.confidence.shape
    assert data_frames == conf_frames, f'Pose {file_name} has different number of frames in data and confidence'
    assert data_people == conf_people, f'Pose {file_name} has different number of people in data and confidence'
    assert data_points == conf_points, f'Pose {file_name} has different number of points in data and confidence'
    assert data_points == 543, f'Pose {file_name} has different number of points in data ({data_points})'
    assert data_dimensions == 3, f'Pose {file_name} has different number of dimensions in data ({data_dimensions})'


def validate_tar_files(tar_file_name):
    """Iterates over the files in a tar archive and validates each one."""
    with tarfile.open(tar_file_name, "r") as tar:
        for member in tqdm(tar.getmembers(), desc="Validating files"):
            f = tar.extractfile(member)
            if f is not None:
                try:
                    validate_pose_file(member.name, f.read())
                except Exception as e:
                    print(f'Error validating {member.name}')
                    print(e)


def list_files_in_tar_gz(file_path):
    with tarfile.open(file_path, "r") as tar:
        file_names = tar.getnames()
        return [os.path.basename(f) for f in file_names]


def main():
    bucket_name = 'sign-mt-poses'
    prefix = 'external/ss'
    blobs = list_blobs_with_prefix(bucket_name, prefix)

    destination_tar = '/home/nlp/amit/WWW/datasets/poses/holistic/signsuisse.tar'
    existing_files = set(list_files_in_tar_gz(destination_tar))
    print(f'Found {len(existing_files)} valid files in {destination_tar}')
    print(list(existing_files)[:10])

    with tarfile.open(destination_tar, "a") as tar:
        for blob in tqdm(blobs, desc="Reading files"):
            if not blob.name.endswith('/'):
                file_name = os.path.basename(blob.name)
                if file_name not in existing_files:
                    download_blob(bucket_name, blob.name, file_name)
                    tar.add(file_name)
                    os.remove(file_name)

    validate_tar_files(destination_tar)


if __name__ == "__main__":
    main()

# Extract: tar xf signsuisse.tar --directory signsuisse
# Compress: tar -cvf signsuisse.tar -C signsuisse .
# import os
#
# from pose_format import Pose
#
# directory = '/home/nlp/amit/WWW/datasets/poses/holistic/signsuisse'
# for filename in tqdm(os.listdir(directory)):
#     if filename.endswith(".pose"):
#         file_path = os.path.join(directory, filename)
#
#         fix_pose_file(file_path)



import csv
import io
import zipfile

import numpy as np
from pose_format.numpy import NumPyPoseBody
from tqdm import tqdm

import webvtt

def convert_time(vtt_time):
    hhmmss, fraction = vtt_time.split('.')
    h, m, s = hhmmss.split(':')
    hhmmss = int(h) * 3600 + int(m) * 60 + int(s)
    return hhmmss + float(int(fraction) / 1000)

def read_pose_tsv_file(pose_tsv: bytes, num_keypoints: int = 33):
    # TSV file where the first cell is the frame ID
    # and the rest of the cells are the pose data (x, y, z, confidence) for every landmark
    rows = pose_tsv.decode('utf-8').strip().split('\n')
    rows = [row.strip().split('\t') for row in rows]

    num_frames = max(int(row[0]) for row in rows)
    tensor = np.zeros((num_frames + 1, num_keypoints, 4), dtype=np.float32)

    for row in rows:
        frame_id = int(row[0])
        pose_data = row[1:]
        if len(pose_data) > 0:
            vec = np.array(pose_data).reshape(-1, 4)
            if len(vec) != num_keypoints:
                print(f"Warning: pose data has wrong number of keypoints ({len(vec)} instead of {num_keypoints})")
            if num_keypoints == 21:  # hands
                vec[:, 3] = 1
            tensor[frame_id] = vec

    return tensor


def load_pose_body_from_zip(zip_file, fps: float) -> NumPyPoseBody:
    face, left_hand, n, body, right_hand = sorted(zip_file.namelist())
    left_hand_tensor = read_pose_tsv_file(zip_file.read(left_hand), 21)
    right_hand_tensor = read_pose_tsv_file(zip_file.read(right_hand), 21)
    body_tensor = read_pose_tsv_file(zip_file.read(body), 33)
    # TODO: we can add the face if needed
    pose_tensor = np.expand_dims(np.concatenate([body_tensor, left_hand_tensor, right_hand_tensor], axis=1), axis=1)

    data = pose_tensor[:, :, :, :3]
    confidence = np.round(pose_tensor[:, :, :, 3])
    return NumPyPoseBody(fps=fps,
                         data=data,
                         confidence=confidence)


def read_mediapi_set(mediapi_path: str, pose_path: str = None, split='test', pose_type="holistic"):
    pose_zip = "mediapipe_zips.zip" if pose_type == "holistic" else "openpose_zips.zip"
    mediapipe_zips = f'mediapi-skel/7/data/{pose_zip}'
    subtitle_zips = 'mediapi-skel/7/data/subtitles.zip'
    information = 'mediapi-skel/7/information/video_information.csv'

    with zipfile.ZipFile(mediapi_path, 'r') as root_zip:
        # Open the information csv using DictReader
        information_info = root_zip.getinfo(information)
        with root_zip.open(information_info, 'r') as information_file:
            text = io.TextIOWrapper(information_file)
            reader = csv.DictReader(text)
            split_data = [row for row in reader if row['train/dev/test'] == split]

            print("Sample test data:", split_data[0])

        name_number = lambda name: int(name.split('/')[-1].split('.')[0]) if not name.endswith('/') else -1

        # Open the subtitles zip and extract the subtitles for every test datum
        subtitle_info = root_zip.getinfo(subtitle_zips)
        with root_zip.open(subtitle_info, 'r') as subtitle_file:
            nested_zip = zipfile.ZipFile(subtitle_file, 'r')
            number_name = {name_number(name): name for name in nested_zip.namelist()}

            for datum in split_data:
                webvtt_text = nested_zip.read(number_name[int(datum['video'])])
                buffer = io.StringIO(webvtt_text.decode('utf-8'))
                datum['subtitles'] = list(webvtt.read_buffer(buffer))
                datum['subtitles'] = [{"start_time": convert_time(c.start), "end_time": convert_time(c.end), "text": c.text} for c in datum["subtitles"]]


            print("Sample subtitle:", split_data[0]['subtitles'])

        # Open the mediapipe zips and extract the mediapipe data for every test datum
        if pose_path is None:
            print("No pose path given. This means the extraction of mediapipe data will be slow. "
                  "Consider extracting the poses zip using the following command:"
                  "unzip -j mediapi-skel.zip mediapi-skel/7/data/mediapipe_zips.zip -d ."
                  "And then pass MEDIAPI_POSE_PATH=mediapipe_zips.zip to this script.")
            mediapipe_info = root_zip.getinfo(mediapipe_zips)
            mediapipe_file = root_zip.open(mediapipe_info, 'r')
            mediapipe_zip = zipfile.ZipFile(mediapipe_file, 'r')
        else:
            mediapipe_zip = zipfile.ZipFile(pose_path, 'r')

        print("mediapipe files", mediapipe_zip.namelist())
        number_name = {name_number(name): name for name in mediapipe_zip.namelist() if '/00001/' not in name}
        for datum in split_data:
            with zipfile.ZipFile(mediapipe_zip.open(number_name[int(datum['video'])]), 'r') as nested_zip:
                datum['id'] = str(datum['video'])
                datum['metadata'] = {
                    'fps': float(datum['fps'].replace(',', '.')),
                    'duration': float(datum['fps'].replace(',', '.')),
                    'frames': int(datum['frames']),
                    'height': int(datum['height']),
                    'width': int(datum['width']),
                }
                if pose_type is not None:
                    try:
                        datum['pose'] = load_pose_body_from_zip(nested_zip, fps=datum['metadata']['fps'])
                    except Exception as e:
                        print("Failed to load pose for", datum['id'], e)
                        print(datum)
                        continue
                yield datum



# import itertools
#
# import tensorflow_datasets as tfds
# from dotenv import load_dotenv
#
# # noinspection PyUnresolvedReferences
# # from sign_language_datasets.datasets.dgs_corpus import DgsCorpusConfig
# from sign_language_datasets.datasets.dgs_corpus import DgsCorpusConfig
#
# import sign_language_datasets.datasets
#
# # noinspection PyUnresolvedReferences
# # import sign_language_datasets.datasets.dgs_corpus
# from sign_language_datasets.datasets.config import SignDatasetConfig
# # from sign_language_datasets.datasets.dgs_corpus.dgs_corpus import DgsCorpusConfig
#
# load_dotenv()
#
# # config = SignDatasetConfig(name="only-annotations", version="3.0.0", include_video=False)
# # rwth_phoenix2014_t = tfds.load(name="rwth_phoenix2014_t", builder_kwargs=dict(config=config))
#
# # config = SignDatasetConfig(name="256x256:10", include_video=True, fps=10, resolution=(256, 256))
#
# # aslg_pc12 = tfds.load('aslg_pc12')
# #
# # rwth_phoenix2014_t = tfds.load('rwth_phoenix2014_t', builder_kwargs=dict(config=config))
#
# # wlasl = tfds.load('wlasl', builder_kwargs=dict(config=config))
# #
# # autsl = tfds.load('autsl', builder_kwargs=dict(
# #     config=SignDatasetConfig(name="test", include_video=False, include_pose="holistic"),
# # ))
#
# # dgs_config = DgsCorpusConfig(name="sentence-test-video", data_type="sentence",
# #                              include_video=False, process_video=False, include_pose=None)
# # dgs_corpus = tfds.load('dgs_corpus', builder_kwargs=dict(config=dgs_config))
# #
# # for datum in itertools.islice(dgs_corpus["train"], 0, 10):
# #   print(datum)
#
#
# config = SignDatasetConfig(name="signbank-annotations", version="1.0.0", include_video=False)
# signbank = tfds.load('sign_bank', builder_kwargs=dict(config=config))
#
# # config = SignDatasetConfig(name="signsuisse3", version="1.0.0", include_video=False, include_pose="holistic")
# # signsuisse = tfds.load('sign_suisse', builder_kwargs=dict(config=config))
#
# # print([d["p.ose"]["data"].shape for d in iter(autsl["train"])])
# # print([d["video"].shape for d in iter(autsl["train"])])
#
# # config = SignDatasetConfig(name="include4", version="1.0.0", extra={"PHPSESSID": "hj9co07ct7f5noq529no9u09l4"})
# # signtyp = tfds.load(name='sign_typ', builder_kwargs=dict(config=config))
# #
# # for datum in itertools.islice(signtyp["train"], 0, 10):
# #   print(datum['sign_writing'].numpy().decode('utf-8'), datum['video'].numpy().decode('utf-8'))
# #
# # config = SignDatasetConfig(name="poses_1", version="1.0.0", include_video=False, include_pose="holistic")
# # dicta_sign = tfds.load(name='dicta_sign', builder_kwargs={"config": config})
#
# # config = SignDatasetConfig(name="only-annotations5", version="1.0.0", include_video=False, process_video=False, include_pose="holistic")
# # dataset = tfds.load(name='sign_bank', builder_kwargs=dict(config=SignDatasetConfig(name="annotations")))
# #
# # decode_str = lambda s: s.numpy().decode('utf-8')
# # for datum in itertools.islice(dataset["train"], 0, 10):
# #     hamnosys = decode_str(datum['hamnosys'])
# #     glosses = [decode_str(g) for g in datum["glosses"]]
# #     print(hamnosys, glosses)
#
#
# #
# # import tensorflow_datasets as tfds
# # # noinspection PyUnresolvedReferences
# # import sign_language_datasets.datasets
# # from sign_language_datasets.datasets.config import SignDatasetConfig
# #
# # # Populate your access tokens
# # TOKENS = {
# #     "zenodo_focusnews_token": "TODO",
# #     "zenodo_srf_videos_token": "TODO",
# #     "zenodo_srf_poses_token": "TODO"
# # }
# #
# # # Load only the annotations, and include path to video files
# # config = SignDatasetConfig(name="annotations", version="1.0.0", process_video=False)
# # wmtslt = tfds.load(name='wmtslt', builder_kwargs={"config": config, **TOKENS})
# #
# # # Load the annotations and openpose poses
# # config = SignDatasetConfig(name="openpose", version="1.0.0", process_video=False, include_pose='openpose')
# # wmtslt = tfds.load(name='wmtslt', builder_kwargs={"config": config, **TOKENS})
# #
# # # Load the annotations and mediapipe holistic poses
# # config = SignDatasetConfig(name="holistic", version="1.0.0", process_video=False, include_pose='holistic')
# # wmtslt = tfds.load(name='wmtslt', builder_kwargs={"config": config, **TOKENS})
# #
# # # Load the full video frames as a tensor
# # config = SignDatasetConfig(name="videos", version="1.0.0", process_video=True)
# # wmtslt = tfds.load(name='wmtslt', builder_kwargs={"config": config, **TOKENS})
# #
# decode_str = lambda s: s.numpy().decode('utf-8')
# for datum in itertools.islice(signsuisse["train"], 0, 10):
#     print(datum)
#     print(datum["pose"])
#     print('\n')
#
#
#

import tensorflow_datasets as tfds
import sign_language_datasets.datasets
from sign_language_datasets.datasets.config import SignDatasetConfig

import itertools
#
# config = SignDatasetConfig(name="holistic-poses", version="3.0.0", include_video=False, include_pose="holistic")
# rwth_phoenix2014_t = tfds.load(name='rwth_phoenix2014_t', builder_kwargs=dict(config=config))
#
# for datum in itertools.islice(rwth_phoenix2014_t["train"], 0, 10):
#     print(datum['gloss'].numpy().decode('utf-8'))
#     print(datum['text'].numpy().decode('utf-8'))
#     print(datum['pose']['data'].shape)
#     print()


config = SignDatasetConfig(name="holistic-poses", version="1.0.0", include_video=False, include_pose="holistic")
mediapi_skel = tfds.load(name='mediapi_skel', builder_kwargs=dict(config=config))

for datum in itertools.islice(mediapi_skel["test"], 0, 10):
    print(datum['id'].numpy().decode('utf-8'))
    print(datum['subtitles'])
    print(datum['pose']['data'].shape)
    print()



"""A Swiss Sign Language Lexicon, combines all three Swiss sign languages: the German-Swiss Sign Language (DSGS), the Langue des Signes Française (LSF) and the Lingua Italiana dei Segni (LIS). ."""
import json
import re
import html

import hashlib
from os import path

import tensorflow_datasets as tfds

from sign_language_datasets.utils.features import PoseFeature
from ..config import cloud_bucket_file

from ..warning import dataset_warning
from ...datasets import SignDatasetConfig

_DESCRIPTION = """
Ein umfangreiches interaktives Lexikon der drei Gebärdensprachen der Schweiz (Deutschschweizerische Gebärdensprache DSGS, Langue des Signes Française LSF und Lingua Italiana dei Segni LIS). Herausgegeben vom Schweizerischen Gehörlosenbund (SGB-FSS)."""

_CITATION = """
@misc{signsuisse,
    title = {{Gehörlosenbund Gebärdensprache-Lexikon}},
    author = {{Schweizerischer Gehörlosenbund SGB-FSS}},
    year = {2023},
    howpublished = {\\url{https://signsuisse.sgb-fss.ch/}},
    note = {Accessed on: \\today}
}
"""

SITE_URL = "https://signsuisse.sgb-fss.ch"

_POSE_URLS = {"holistic": cloud_bucket_file("poses/holistic/signsuisse.tar")}
_POSE_HEADERS = {"holistic": path.join(path.dirname(path.realpath(__file__)), "holistic.poseheader")}


class SignSuisse(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for signsuisse dataset."""

    MAX_SIMULTANEOUS_DOWNLOADS = 1  # SignSuisse allows for a maximum of 1 simultaneous connection
    VERSION = tfds.core.Version("1.0.0")
    RELEASE_NOTES = {
        "1.0.0": "Initial crawl.",
    }

    BUILDER_CONFIGS = [
        SignDatasetConfig(name="default", include_video=False),
        SignDatasetConfig(name="holistic", include_video=False, include_pose='holistic'),
    ]

    def _info(self) -> tfds.core.DatasetInfo:
        """Return s the dataset metadata."""

        features = {
            "id": tfds.features.Text(),
            "name": tfds.features.Text(),
            "category": tfds.features.Text(),
            "spokenLanguage": tfds.features.Text(),
            "signedLanguage": tfds.features.Text(),
            "url": tfds.features.Text(),
            "paraphrase": tfds.features.Text(),
            "definition": tfds.features.Text(),
            "exampleText": tfds.features.Text(),
        }

        if self._builder_config.include_video:
            raise ValueError("To prevent spamming signSuisse with requests, include_video=True is not allowed.")

        if self._builder_config.include_video and self._builder_config.process_video:
            features["video"] = self._builder_config.video_feature((640, 480))
            features["exampleVideo"] = self._builder_config.video_feature((640, 480))
        else:
            features["video"] = tfds.features.Text()
            features["exampleVideo"] = tfds.features.Text()

        if self._builder_config.include_pose == "holistic":
            pose_header_path = _POSE_HEADERS[self._builder_config.include_pose]
            stride = 1 if self._builder_config.fps is None else 25 / self._builder_config.fps
            features["pose"] = PoseFeature(shape=(None, 1, 543, 3),
                                           header_path=pose_header_path,
                                           stride=stride,
                                           include_path=True)
            features["examplePose"] = features["pose"]

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage=SITE_URL,
            supervised_keys=None,
            citation=_CITATION,
        )

    def _list_all_lexicon_items(self, dl_manager: tfds.download.DownloadManager):
        languages = ['de', 'fr', 'it']
        dl_links = [f"https://signsuisse.sgb-fss.ch/index.php?eID=sitemap&lang={l}&format=json"
                    for l in languages]
        # FYI: to reset the indexes, run `rm $(ls *sitemap*)` in the `tensorflow_datasets/downloads/` folder

        indexes = dl_manager.download(dl_links)
        results = []
        for index in indexes:
            with open(index, "r", encoding="utf-8") as f:
                index = json.load(f)
            results.extend(index)

        if self._builder_config.sample_size is not None:
            return results[:self._builder_config.sample_size]
        return results

    def _parse_html(self, item_name: str, item_page: str):
        with open(item_page, "r", encoding="utf-8") as f:
            html_content = f.read()

        # Verify that the page matches the item
        title = re.search(r"<h1.*?>(.*?)</h1>", html_content).group(1).strip()
        title = html.unescape(title)
        while '  ' in title:
            title = title.replace('  ', ' ')

        if title != item_name:
            raise ValueError(f"Title does not match item name '{item_name}' != '{title}'")

        example = re.search(r"(Beispiel|Exemple|Esempio)</h2>\s*<p>(.*?)</p>[\s\S]*?<video[\s\S]*?src=\"(.*?)\"",
                            html_content)
        if example is not None:
            example_text = example.group(2).strip()
            example_video = SITE_URL + example.group(3).strip()
        else:
            # This happens on: https://signsuisse.sgb-fss.ch/de/lexikon/g/shakespeare-william/
            # Because the page is formatted differently. I requested a change to the page.
            example_text = ""
            example_video = ""

        video = SITE_URL + re.search(r"<video id=\"video-main\"[\s\S]*?src=\"(.*?)\"", html_content).group(1).strip()
        paraphrase_match = re.search(
            r"(Umschreibung|Périphrase / Synonyme|Parafrasi \(descrizione\))</h2>\s*<p>(.*?)</p>", html_content)
        paraphrase = paraphrase_match.group(2).strip() if paraphrase_match else ""
        definition_match = re.search(r"(Definition|Definizione|Définition)</h2>\s*<p>(.*?)</p>", html_content)
        definition = definition_match.group(2).strip() if definition_match else ""
        category_match = re.search(r"<strong>(Kategorien|Categoria|Catégorie):</strong>[\s\S]*?<span>([\s\S]*?)</span",
                                   html_content)
        category = category_match.group(2).strip() if category_match else ""

        return {
            "category": category,
            "paraphrase": paraphrase,
            "definition": definition,
            "exampleText": example_text,
            "exampleVideo": example_video,
            "video": video
        }

    def _parse_item(self, item, item_page=None):
        item["name"] = item["name"].replace("   ", " ").replace("  ", " ")

        if item_page is not None:
            item_values = self._parse_html(item["name"], item_page)
        else:
            item_values = {
                "category": item["kategorie"],
                "paraphrase": "",
                "definition": "",
                "exampleText": "",
                "exampleVideo": "",
                "video": ""
            }

        return {
            "id": item["uid"],
            "name": item["name"],
            "spokenLanguage": item["sprache"],
            "signedLanguage": "ch-" + item["sprache"],
            "url": item["link"],
            **item_values
        }

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        dataset_warning(self)
        print(
            "The lexicon is available free of charge, so we look forward to your donation! https://www.sgb-fss.ch/spenden/jetzt-spenden/")

        lexicon_items = self._list_all_lexicon_items(dl_manager)
        print("Found", len(lexicon_items), "lexicon items.")
        item_urls = [item["link"] for item in lexicon_items]
        if self._builder_config.include_video:
            items_pages = dl_manager.download(item_urls)
        else:
            items_pages = [None for _ in item_urls]

        data = []
        for item, item_page in zip(lexicon_items, items_pages):
            try:
                item = self._parse_item(item, item_page)
                if item is not None:
                    data.append(item)
            except Exception as e:
                print("Failed to parse item")
                print(item)
                print(item_page)
                print(e)
                print("\n\n\n\n\n\n")
                # raise e

        # Download videos if requested
        if self._builder_config.include_video:
            video_urls = [item["video"] for item in data]
            videos = dl_manager.download(video_urls)
            for datum, video in zip(data, videos):
                datum["video"] = video  # PosixPath
                if not self._builder_config.process_video:
                    datum["video"] = str(datum["video"])

            # Download example videos if requested
            data_with_examples = [item for item in data if item["exampleVideo"] != ""]
            video_urls = [item["exampleVideo"] for item in data_with_examples]
            videos = dl_manager.download(video_urls)
            for datum, video in zip(data_with_examples, videos):
                datum["exampleVideo"] = video  # PosixPath
                if not self._builder_config.process_video:
                    datum["exampleVideo"] = str(datum["exampleVideo"])

        if self._builder_config.include_pose is not None:
            poses_dir = dl_manager.download_and_extract(_POSE_URLS[self._builder_config.include_pose])
            id_func = lambda opt: 'ss' + hashlib.md5(("signsuisse" + opt[0] + opt[1]).encode()).hexdigest()

            for datum in data:
                pose_file = poses_dir.joinpath(id_func([datum["id"], "isolated"]) + ".pose")
                datum["pose"] = pose_file if pose_file.exists() else None

                if datum["exampleVideo"] != "":
                    pose_file = poses_dir.joinpath(id_func([datum["id"], "example"]) + ".pose")
                    datum["examplePose"] = pose_file if pose_file.exists() else None
                else:
                    datum["examplePose"] = None

        return {"train": self._generate_examples(data)}

    def _generate_examples(self, data):
        for datum in data:
            yield datum["id"], datum



from pose_format import PoseHeader
from pose_format.pose_header import PoseHeaderDimensions


def get_pose_header():
    from pose_format.utils.holistic import holistic_components

    pose_components = [c for c in holistic_components()
                       if c.name in ["POSE_LANDMARKS", "LEFT_HAND_LANDMARKS", "RIGHT_HAND_LANDMARKS"]]
    dimensions = PoseHeaderDimensions(width=1, height=1, depth=1)
    return PoseHeader(version=0.1, dimensions=dimensions, components=pose_components)


if __name__ == "__main__":
    pose_header = get_pose_header()
    with open("holistic.header", "wb") as f:
        pose_header.write(f)



"""MEDIAPI-SKEL is a bilingual corpus of French Sign Language (LSF) in the form of pose data (coordinates of skeleton points) and French in the form of subtitles. It can be used for academic research."""
import os
from os import path

import tensorflow as tf
import tensorflow_datasets as tfds

from .mediapi_utils import read_mediapi_set
from ..warning import dataset_warning
from ...datasets.config import SignDatasetConfig
from ...utils.features import PoseFeature

_DESCRIPTION = """
MEDIAPI-SKEL is a bilingual corpus of French Sign Language (LSF) in the form of pose data (coordinates of skeleton points) and French in the form of subtitles. It can be used for academic research.

To constitute this corpus, we used 368 subtitled videos totaling 27 hours of LSF footage and written French produced by Média'Pi!, a media company producing bilingual content with LSF and written French .

We cannot provide the original videos, which can be accessed via a subscription to Média'Pi!. We provide the French subtitles an WebVTT format, aligned with two types of data: openpose and mediapipe holistic (in version 2 of the repository), calculated from each of the videos.

Data des cription:
X corresponds to the number of the corresponding video, from 00001 to 00368.
- Subtitles: For each X video, we provide a X.fr.vtt file containing the corresponding French subtitles.
- OpenPose: For each X video, we supply a X.zip archive containing as many json files as there are frames in the video, named X_Y_keypoints.json, where Y is the frame number. Each json file contains the coordinates of 137 points (25 for the body, 2x21 for the hands and 70 for the face).
- Mediapipe Holistic: For each X video, we supply a X.zip archive containing 5 csv files. X_N contains the image size (width and height), X_left_hand, X_right_hand, X_face and X_body contain the data for the hands, face and body respectively, with 1 line per frame. The 1st column shows the frame number, then for each point on the skeleton (21 for each hand, 468 for the face and 33 for the body), there is the following information: x (in pixels), y (in pixels), z (between -1 and 0), visibility (0 or 1 depending on whether the point is visible or not).

Directories and files:
Data directory
- openpose_zips.zip
- mediapipe_zips.zip
- subtitles.zip
Information directory
- video_information.csv: contains information about the resolution, frame rate, duration and number of frames of each video, as well as a breakdown of videos in train/dev/test sets.
- example: directory containing a short extract
MEDIAPI-SKEL is a 2D-skeleton video corpus of French Sign Language (LSF) with French subtitles. It can be used by members of public university and research structures.
"""

_CITATION = """
@inproceedings{bull-etal-2020-mediapi,
    title = "{MEDIAPI}-{SKEL} - A 2{D}-Skeleton Video Database of {F}rench {S}ign {L}anguage With Aligned {F}rench Subtitles",
    author = "Bull, Hannah  and
      Braffort, Annelies  and
      Gouiff{\`e}s, Mich{\`e}le",
    booktitle = "Proceedings of the Twelfth Language Resources and Evaluation Conference",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2020.lrec-1.743",
    pages = "6063--6068",
    language = "English",
    ISBN = "979-10-95546-34-4",
}
"""

_POSE_HEADERS = {
    "holistic": path.join(path.dirname(path.realpath(__file__)), "holistic.header"),
    "openpose": path.join(path.dirname(path.realpath(__file__)), "openpose.header")
}

_SPLITS = {
    tfds.Split.TRAIN: "train",
    tfds.Split.VALIDATION: "dev",
    tfds.Split.TEST: "test",
}


class MediapiSkel(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for mediapi_skel dataset."""

    VERSION = tfds.core.Version("1.0.0")
    RELEASE_NOTES = {"1.0.0": "Initial release."}

    BUILDER_CONFIGS = [SignDatasetConfig(name="default", include_pose="openpose")]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "id": tfds.features.Text(),
            "metadata": {
                "fps": tf.int32,
                "height": tf.int32,
                "width": tf.int32,
                "duration": tf.float32,
                "frames": tf.int32,
            },
            "subtitles": tfds.features.Sequence({
                "start_time": tf.float32,
                "end_time": tf.float32,
                "text": tfds.features.Text(),
            }),
        }

        if self._builder_config.include_pose is not None:
            pose_header_path = _POSE_HEADERS[self._builder_config.include_pose]
            if self._builder_config.fps is not None:
                print("Pose FPS is not implemented for mediapi_skel dataset (since the original fps is not consistent)")

            if self._builder_config.include_pose == "openpose":
                pose_shape = (None, 1, 137, 2)
                raise NotImplementedError("Openpose is available, but not yet implemented for mediapi_skel dataset.")

            if self._builder_config.include_pose == "holistic":
                pose_shape = (None, 1, 75, 3)

            features["pose"] = PoseFeature(shape=pose_shape, stride=1, header_path=pose_header_path)

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage="https://www.ortolang.fr/market/corpora/mediapi-skel/v2",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        ZIP_PATH = os.environ.get("MEDIAPI_PATH", None)
        if ZIP_PATH is None:
            raise ValueError("MEDIAPI_PATH environment variable not set.")
        if not os.path.exists(ZIP_PATH):
            raise ValueError(f"MEDIAPI_PATH environment variable points to non-existent path: {ZIP_PATH}")

        if self._builder_config.include_video:
            raise NotImplementedError("Video does not exist for mediapi_skel dataset.")

        POSE_ZIP_PATH = os.environ.get("MEDIAPI_POSE_PATH", None)
        if POSE_ZIP_PATH is not None and not os.path.exists(POSE_ZIP_PATH):
            raise ValueError(f"MEDIAPI_POSE_PATH environment variable points to non-existent path: {POSE_ZIP_PATH}")

        return [
            tfds.core.SplitGenerator(name=name,
                                     gen_kwargs={"data": read_mediapi_set(mediapi_path=ZIP_PATH,
                                                                          pose_path=POSE_ZIP_PATH,
                                                                          split=split,
                                                                          pose_type=self._builder_config.include_pose)})
            for name, split in _SPLITS.items()
        ]

    def _generate_examples(self, data: iter):
        """ Yields examples. """

        for datum in data:
            yield datum["id"], {
                "id": datum["id"],
                "metadata": datum["metadata"],
                "subtitles": datum["subtitles"],
                "pose": datum["pose"]
            }



# from pose_format import Pose
# from sign_language_datasets.datasets.dgs_corpus.dgs_corpus import _POSE_HEADERS
# from sign_language_datasets.utils.features import PoseFeature
#
# files = [
#     "/home/nlp/amit/tensorflow_datasets/downloads/nlp.biu.ac.il_amit_data_pose_holi_dgs_coreCZFUT1DjfgqeTJH8Be7LXAdZjfIcl6MFu9-a0QsMiM.pose",
#     f"/home/nlp/amit/WWW/datasets/poses/holistic/dgs_corpus/1248625-15324720-15465943_a.pose",
#     "/home/nlp/amit/tensorflow_datasets/downloads/nlp.biu.ac.il_amit_data_pose_holi_dgs_corQoeprd_NyhZVoKS6jLUg0R4Yjart_u7A0LK2dpvd2f4.pose",
#     f"/home/nlp/amit/WWW/datasets/poses/holistic/dgs_corpus/1248625-15324720-15465943_b.pose",
# ]
#
# for f_name in files:
#     with open(f_name, "rb") as f:
#         pose = Pose.read(f.read())
#
#     print("fps", pose.body.fps)
#     print("length", len(pose.body.data))
#
#     stride = 50 / 25
#     pose_feature = PoseFeature(shape=(None, 1, 543, 3), stride=stride, header_path=_POSE_HEADERS["holistic"])
#
#     encoded = pose_feature.encode_example(pose)
#     print("fps", encoded["fps"])
#     print("shape", encoded["data"].shape)
#     print("")
#     # [00:32<04:31,  1.90it/s]{'id': '1247205_b', 'fps': 25, 'shape': (868, 1, 75, 3), 'length': 868, 'timestamps': tensor(34.6800)}
#
#     # fps 50
#     # length 42623
#     # fps 25
#     # shape (21312, 1, 543, 3)
#     #
#     # fps 50
#     # length 42623
#     # fps 25
#     # shape (21312, 1, 543, 3)
import json
import os

directory = "/home/nlp/amit/WWW/datasets/poses/holistic/dgs_corpus/"

# Get list of files in the directory
files = os.listdir(directory)

# Create empty dictionary
file_sizes = {}

# Loop through each file in the directory
for file in files:
    # Get full file path
    file_path = os.path.join(directory, file)

    # Get size of file in bytes
    file_size = os.path.getsize(file_path)

    # Add file name and size to dictionary
    file_sizes["https://nlp.biu.ac.il/~amit/datasets/poses/holistic/dgs_corpus/" + file] = file_size

# Print the dictionary
print(file_sizes)

json.dump(file_sizes, open("holistic_pose_size.json", "w"))


import tensorflow_datasets as tfds

# noinspection PyUnresolvedReferences
from pose_format import PoseHeader, Pose
from pose_format.numpy import NumPyPoseBody
from pose_format.utils.reader import BufferReader

from sign_language_datasets.datasets.signsuisse.signsuisse import _POSE_HEADERS
from sign_language_datasets.datasets.config import SignDatasetConfig


def load_pose(pose_header, tf_pose):
    fps = int(tf_pose["fps"].numpy())
    pose_body = NumPyPoseBody(fps, tf_pose["data"].numpy(), tf_pose["conf"].numpy())
    return Pose(pose_header, pose_body)


config = SignDatasetConfig(name="example", version="1.0.0",
                           include_video=True,
                           include_pose="holistic",
                           process_video=False,
                           sample_size=10)
dataset = tfds.load(name='sign_suisse', builder_kwargs={"config": config})

# pylint: disable=protected-access
with open(_POSE_HEADERS[config.include_pose], "rb") as buffer:
    pose_header = PoseHeader.read(BufferReader(buffer.read()))

for datum in dataset["train"]:
    print("Concept: ", datum['name'].numpy().decode('utf-8'))
    print("category: ", datum['category'].numpy().decode('utf-8'))
    print("Video: ", datum['video'].numpy().decode('utf-8'))
    print("Pose path: ", datum['pose']['path'].numpy().decode('utf-8'))
    print("Pose: ", load_pose(pose_header, datum['pose']))
    print("---")
    print("Example: ", datum['exampleText'].numpy().decode('utf-8'))
    print("Example Video: ", datum['exampleVideo'].numpy().decode('utf-8'))
    print("Example Pose path: ", datum['examplePose']['path'].numpy().decode('utf-8'))
    print("Example Pose: ", load_pose(pose_header, datum['examplePose']))
    print("---")
    print('\n\n')



"""The Multilingual Sign Language Wordnet"""
import csv

import tensorflow_datasets as tfds

from ..warning import dataset_warning
from ...datasets import SignDatasetConfig

_DESCRIPTION = """
This is an ongoing project to add several European sign languages to the Open Multilingual Wordnet. It is part of the ongoing EASIER project, whose goal is to create an intelligent translation framework for sign languages. This Wordnet aims at connecting signs with a shared meaning accross languages.

The current available version covers German Sign Language, Greek Sign Language, British Sign Language, Sign Language of the Netherlands, and French Sign Language. More than 9 000 signs have been successfully linked to one or more synsets, with 15 000 more still being processed. It is a work in progress, and the coverage is limited.
"""

_CITATION = """
@inproceedings{Bigeard2022IntroducingSL,
  title={Introducing Sign Languages to a Multilingual Wordnet: Bootstrapping Corpora and Lexical Resources of Greek Sign Language and German Sign Language},
  author={Sam Bigeard and Marc Schulder and Maria Kopf and Thomas Hanke and Kyriaki Vasilaki and Anna Vacalopoulou and Theodoros Goulas and Lida Dimou and Evita F. Fotinea and Eleni Efthimiou},
  booktitle={SIGNLANG},
  year={2022},
  url={https://api.semanticscholar.org/CorpusID:250086794}
}
"""

# Map between Sign Wornet language codes and IANA language codes
IANA_MAP = {
    "bsl": "bfi",
    "dgs": "gsg",
    "gsl": "gss",
    "lsf": "fsl",
    "ngt": "dse"
}


def no_space(d: dict):
    for k, v in d.items():
        if isinstance(v, str):
            d[k] = v.strip()
    return d

class SignWordnet(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for SignWordnet dataset."""

    VERSION = tfds.core.Version("0.2.0")
    RELEASE_NOTES = {
        "0.2.0": "Initial release.",
    }

    BUILDER_CONFIGS = [
        SignDatasetConfig(name="default", include_video=True)
    ]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "id": tfds.features.Text(),
            "sign_language": tfds.features.Text(),
            "sign_language_name": tfds.features.Text(),
            "translations": tfds.features.Sequence({
                "language": tfds.features.Text(),
                "text": tfds.features.Text()
            }),
            "links": tfds.features.Text(),
            "gloss": tfds.features.Text(),
            "video": tfds.features.Text(),
            "synset_id": tfds.features.Text(),
            "confidence": tfds.features.Text(),
        }

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage="https://www.sign-lang.uni-hamburg.de/easier/sign-wordnet/index.html",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        try:
            import nltk
        except ImportError:
            raise ImportError("Please install nltk with: pip install nltk")

        nltk.download("wordnet")
        nltk.download("omw-1.4")
        nltk.download("extended_omw")

        signs_data_url = "https://www.sign-lang.uni-hamburg.de/easier/sign-wordnet/static/csv/sign_data.csv"
        accepted_synsets_url = "https://www.sign-lang.uni-hamburg.de/easier/sign-wordnet/static/csv/synset_links_accepted.csv"
        custom_synsets_url = "https://www.sign-lang.uni-hamburg.de/easier/sign-wordnet/static/csv/custom_synsets.csv"

        signs_data, accepted_synsets, custom_synsets = dl_manager.download(
            [signs_data_url, accepted_synsets_url, custom_synsets_url])

        return {
            "train": self._generate_examples(signs_data, accepted_synsets, custom_synsets),
        }

    def _generate_examples(self, signs_data: str, accepted_synsets: str, custom_synsets: str):
        from nltk.corpus import wordnet as wn
        # List all available languages in OMW
        languages = wn.langs()
        print('Translation Languages', languages)

        with open(custom_synsets, "r", encoding="utf-8") as csvfile:
            custom_synsets_dict = {row["synset_id"]: no_space(row) for row in csv.DictReader(csvfile)}

        with open(signs_data, "r", encoding="utf-8") as csvfile:
            signs_data_dict = {row["sign_id_wordnet"]: no_space(row) for row in csv.DictReader(csvfile)}

        with open(accepted_synsets, "r", encoding="utf-8") as csvfile:
            accepted_synsets_dict = {row["sign_id"]: no_space(row) for row in csv.DictReader(csvfile)}

        for _id, row in signs_data_dict.items():
            accepted_synset = accepted_synsets_dict.get(_id, {})
            synset_id = accepted_synset.get("synset_id", "")  # Looks like  omw.00672433-v
            translations = []

            if synset_id != "":  # Looks like  omw.00672433-v
                if synset_id in custom_synsets_dict:
                    translations.append({
                        "language": "en",
                        "text": custom_synsets_dict[synset_id]["words_english"]
                    })
                else:
                    # TODO: track open issue https://github.com/omwn/omw-data/issues/35
                    _, omw_id = synset_id.split('.')
                    omw_offset, omw_pos = omw_id.split('-')
                    synset = wn.synset_from_pos_and_offset(omw_pos, int(omw_offset))

                    # Get translations in all languages
                    for lang in languages:
                        lemmas = synset.lemmas(lang=lang)
                        if lemmas:  # If there are lemmas for this language
                            for lemma in lemmas:
                                translations.append({"language": lang, "text": lemma.name()})

            yield _id, {
                "id": _id,
                "sign_language": IANA_MAP[row["language"]],
                "sign_language_name": row["language"],
                "translations": translations,
                "links": row["links"],
                "gloss": row["gloss"],
                "video": row["video"],
                "synset_id": accepted_synset.get("synset_id", ""),
                "confidence": accepted_synset.get("confidence", ""),
            }



"""SignBank dataset."""
import re
import xml.etree.ElementTree as ET
from datetime import datetime
from typing import List

import tensorflow as tf
import tensorflow_datasets as tfds
from tqdm import tqdm

from ..warning import dataset_warning
from ...datasets import SignDatasetConfig

_DESCRIPTION = """
SignBank Site: SignWriting Software for Sign Languages, including SignMaker 2017, 
SignPuddle Online, the SignWriting Character Viewer, SignWriting True Type Fonts, 
Delegs SignWriting Editor, SignBank Databases in FileMaker, SignWriting DocumentMaker, 
SignWriting Icon Server, the International SignWriting Alphabet (ISWA 2010) HTML Reference Guide, 
the ISWA 2010 Font Reference Library and the RAND Keyboard for SignWriting.
"""

# TODO(SignBank): BibTeX citation
_CITATION = """
"""


def is_signwriting(fsw: str) -> bool:
    return bool(re.match(r"[A]?[abcdef0-9S]*?([BLMR])(\d{3})x(\d{3})", fsw))


PUDDLES = {
    2: ["my", "mm", "ysm"],  # Myanmar Dictionary
    4: ["en", "us", "ase"],  # Dictionary US
    5: ["en", "us", "ase"],  # Literature US
    11: ["en", "sg", "sls"],  # Singapore Sign Language (SgSL) Dictionary
    12: ["zh-CN", "hk", "hks"],  # Hong Kong Dictionary
    13: ["zh-CN", "hk", "hks"],  # Hong Kong Literature
    14: ["en", "sg", "sls"],  # Singapore Sign Language (SgSL) Literature
    16: ["es", "hn", "hds"],  # Diccionario Honduras
    17: ["en", "us", "ase"],  # Deaf Harbor
    18: ["am", "et", "eth"],  # Dictionary Ethiopia
    19: ["pl", "pl", "pso"],  # Słownik PL
    20: ["fr", "ch", "ssr"],  # Littérature CH-fr
    21: ["en", "us", "ase"],  # Encyclopedia US
    22: ["fr", "ch", "ssr"],  # Encyclopédie CH-fr
    23: ["no", "no", "nsl"],  # Litteratur NO
    24: ["no", "no", "nsl"],  # Leksikon NO
    25: ["en", "us", "ase"],  # LLCN & SignTyp
    26: ["de", "de", "gsg"],  # Literatur DE
    27: ["de", "de", "gsg"],  # Enzyklopädie DE
    28: ["en", "us", "ase"],  # ASL Bible Dictionary
    29: ["de", "at", "asq"],  # Wörterbuch AT
    30: ["da", "dk", "dsl"],  # Ordbog Danmark
    31: ["mt", "mt", "mdl"],  # Dictionary Malta
    32: ["en", "ng", "nsi"],  # Dictionary Nigeria
    33: ["pt", "pt", "psr"],  # Dicionário Portugal
    34: ["th", "th", "tsq"],  # Dictionary Thailand
    35: ["en", "isl", "ase"],  # Dictionary International
    36: ["cs", "cz", "cse"],  # Literatura CZ
    37: ["cs", "cz", "cse"],  # Encyklopedie CZ
    38: ["pl", "pl", "pso"],  # Literatura PL
    39: ["pl", "pl", "pso"],  # Encyklopedie PL
    40: ["ar", "sa", "sdl"],  # Dictionary Saudi Arabia
    41: ["es", "ar", "aed"],  # Diccionario Argentina
    42: ["en", "au", "asf"],  # Dictionary Australia
    43: ["fr", "be", "sfb"],  # Dictionnaire BE-fr
    44: ["nl", "be", "vgt"],  # Woordenboek Flanders
    45: ["es", "bo", "bvl"],  # Diccionario Bolivia
    46: ["pt", "br", "bzs"],  # Dicionário Brasil
    47: ["fr", "ca", "fcs"],  # Dictionnaire Quebec
    48: ["de", "ch", "sgg"],  # Wörterbuch CH-de
    49: ["fr", "ch", "ssr"],  # Dictionnaire CH-fr
    50: ["it", "ch", "slf"],  # Dizionario CH-it
    51: ["es", "co", "csn"],  # Diccionario Colombia
    52: ["sk", "sk", "svk"],  # Slovník CZ
    53: ["de", "de", "gsg"],  # Wörterbuch DE
    54: ["eo", "", "ase"],  # Vortaro (esperanto)
    55: ["es", "es", "ssp"],  # Diccionario España
    56: ["ca", "es", "csc"],  # Diccionario Catalán
    57: ["fi", "fi", "fse"],  # Dictionary Finland
    58: ["fr", "fr", "fsl"],  # Dictionnaire FR
    59: ["en", "gb", "bfi"],  # Dictionary Great Britain
    60: ["en", "ie", "isg"],  # Dictionary Northern Ireland
    61: ["gr", "gr", "gss"],  # Dictionary Greece
    62: ["en", "ir", "psc"],  # Dictionary Ireland
    63: ["it", "it", "ise"],  # Dizionario IT
    64: ["ja", "jp", "jsl"],  # Dictionary Japan
    65: ["es", "mx", "mfs"],  # Diccionario Mexico
    66: ["ms", "my", "xml"],  # Dictionary Malaysia
    67: ["es", "ni", "ncs"],  # Diccionario Nicaragua
    68: ["nl", "nl", "dse"],  # Woordenboek NL
    69: ["no", "no", "nsl"],  # Ordbok NO
    70: ["en", "nz", "nzs"],  # Dictionary New Zealand
    71: ["es", "pe", "prl"],  # Diccionario Peru
    72: ["fil", "ph", "psp"],  # Dictionary Philippines
    73: ["sv", "se", "swl"],  # Ordbok Sverige
    74: ["sl", "sl", "ysl"],  # Slovar Slovenia
    75: ["zh-TW", "tw", "tss"],  # Dictionary Taiwan
    76: ["es", "ve", "vsl"],  # Diccionario Venezuela
    77: ["en", "za", "sfs"],  # Dictionary South Africa
    78: ["ko", "kr", "kvk"],  # Dictionary Korea
    79: ["sw", "ke", "xki"],  # Dictionary Kenya
    80: ["pt", "pt", "psr"],  # Project 2 Dictionary Sorting
    81: ["fr", "ca", "fcs"],  # Littérature Quebec
    82: ["sq", "al", "sqk"],  # Dictionary Albania
    83: ["zh-CN", "cn", "csl"],  # Dictionary China
    84: ["ar", "eg", "esl"],  # Dictionary Egypt
    85: ["hi", "in", "ins"],  # Dictionary India
    86: ["ar", "jo", "jos"],  # Dictionary Jordan
    87: ["ur", "pk", "pks"],  # Dictionary Pakistan
    88: ["ru", "ru", "rsl"],  # Dictionary Russia
    89: ["sk", "sk", "svk"],  # Dictionary Slovakia
    90: ["tr", "tr", "tsm"],  # Dictionary Turkey
    91: ["ar", "sa", "sdl"],  # Literature Saudi Arabia
    92: ["ar", "jo", "jos"],  # Literature Jordan
    93: ["es", "es", "ssp"],  # Literatura España
    94: ["ca", "es", "csc"],  # Literatura Catalán
    95: ["fr", "be", "sfb"],  # Littérature BE-fr
    96: ["de", "ch", "sgg"],  # Literatur CH-de
    98: ["nl", "be", "vgt"],  # Literatuur Flanders
    99: ["ja", "jp", "jsl"],  # Literature Japan
    100: ["am", "et", "eth"],  # Literature Ethiopia
    103: ["mt", "mt", "mdl"],  # Malta LSM Private Puddle
    104: ["ar", "tn", "tse"],  # Dictionnaire Tunisien
    105: ["en", "us", "ase"],  # DAC Private Puddle
    106: ["ps", "af", "afg"],  # Dictionary Afghanistan
    107: ["lt", "lt", "lls"],  # Dictionary Lithuania
    108: ["lv", "lv", "lsl"],  # Dictionary Latvia
    109: ["et", "et", "eso"],  # Dictionary Estonia
    110: ["he", "il", "isr"],  # Dictionary Israel
    111: ["en", "us", "ase"],  # Project 1 Translate Wiki
    112: ["es", "gt", "gsm"],  # Dictionary Guatemala
    113: ["ht", "ht", ""],  # Dictionary Haiti
    114: ["pt", "br", "bzs"],  # Literatura Brasil
    115: ["pt", "pt", "psr"],  # Literatura Portugal
    116: ["pt", "br", "bzs"],  # Enciclopédia Brasil
    117: ["pt", "pt", "psr"],  # Enciclopédia Portugal
    118: ["da", "dk", "dsl"],  # Litteratur Danmark
    119: ["es", "ni", "ncs"],  # Literatura Nicaragua
    120: ["es", "mx", "mfs"],  # Literatura Mexico
    122: ["hu", "hu", "hsh"],  # Dictionary Hungary
    123: ["hu", "hu", "hsh"],  # Literature Hungary
    124: ["fr", "fr", "fsl"],  # Literature France
    125: ["en", "gb", "bfi"],  # Literature Great Britain
    126: ["ar", "tn", "tse"],  # Littérature Tunisien
    127: ["mt", "mt", "mdl"],  # Literature Malta
    128: ["mw", "mw", "lws"],  # Dictionary Malawi
    129: ["gn", "py", "pys"],  # Diccionario Paraguay
    130: ["uk", "ua", "ukl"],  # Dictionary Ukraine
    131: ["is", "is", "icl"],  # Ordabók IS
    132: ["ro", "ro", "rms"],  # Dictionary Romania
    133: ["ne", "np", "nsp"],  # Dictionary Nepal
    134: ["bg", "bg", "bqn"],  # Dictionary Bulgaria
    135: ["es", "cl", "csg"],  # Diccionario Chile
    136: ["es", "ec", "ecs"],  # Diccionario Ecuador
    137: ["es", "sv", "esn"],  # Diccionario El Salvador
    138: ["ro", "ro", "rms"],  # Literature Romania
    139: ["ro", "ro", "rms"],  # Encyclopedia Romania
    140: ["fr", "ca", "fcs"],  # Encyclopédie Quebec
    141: ["ru", "ru", "rsl"],  # Literature Russia
    142: ["ru", "ru", "rsl"],  # Encyclopedia Russia
    143: ["es", "uy", "ugy"],  # Diccionario Uruguay
    144: ["es", "uy", "ugy"],  # Literatura Uruguay
    145: ["es", "ar", "aed"],  # Literatura Argentina
    146: ["es", "ar", "aed"],  # Enciclopedia Argentina
    147: ["mt", "mt", "mdl"],  # Literature Malta Archive
    148: ["sl", "sl", "ysl"],  # Besedila Slovenia
    149: ["sl", "sl", "ysl"],  # Enciklopedija Slovenia
    150: ["", "", ""],  # Anthropology Book Project
    151: ["en", "us", "ase"],  # ASL Bible Books NLT
    152: ["en", "us", "ase"],  # ASL Bible Books Shores Deaf Church
    153: ["vn", "vn", "haf"],  # Dictionary Vietnam
}

CACHE_BUSTER = str(datetime.today().date())

class SignBank(tfds.core.GeneratorBasedBuilder):
    """DatasetBuilder for SignBank dataset."""

    VERSION = tfds.core.Version("1.0.0")
    RELEASE_NOTES = {
        "1.0.0": "Initial release.",
    }

    BUILDER_CONFIGS = [SignDatasetConfig(name="default", include_video=False)]

    def _info(self) -> tfds.core.DatasetInfo:
        """Returns the dataset metadata."""

        features = {
            "puddle": tf.int32,
            "id": tfds.features.Text(),
            "assumed_spoken_language_code": tfds.features.Text(),
            "sign_language_code": tfds.features.Text(),
            "country_code": tfds.features.Text(),
            "created_date": tfds.features.Text(),
            "modified_date": tfds.features.Text(),
            "sign_writing": tfds.features.Sequence(tfds.features.Text()),
            "terms": tfds.features.Sequence(tfds.features.Text()),
            "user": tfds.features.Text(),
        }

        return tfds.core.DatasetInfo(
            builder=self,
            description=_DESCRIPTION,
            features=tfds.features.FeaturesDict(features),
            homepage="http://signbank.org/",
            supervised_keys=None,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: tfds.download.DownloadManager):
        """Returns SplitGenerators."""
        dataset_warning(self)

        index = dl_manager.download("http://signbank.org/signpuddle2.0/data/spml/")
        regex = r"\"sgn[\d]+.spml\""
        with open(index, "r", encoding="utf-8") as f:
            matches = [match.group()[1:-1] for match in re.finditer(regex, f.read(), re.MULTILINE)]
            spml_urls = ["http://signbank.org/signpuddle2.0/data/spml/" + match + "?buster=" + CACHE_BUSTER
                         for match in matches if match != "sgn0.spml"]

        spmls = dl_manager.download(spml_urls)

        return {
            "train": self._generate_examples(spmls),
        }

    def _generate_examples(self, spmls: List[str]):
        """Yields examples."""

        i = 0
        for spml in tqdm(reversed(spmls)):
            tree = ET.parse(spml)
            root = tree.getroot()

            puddle = int(root.attrib["puddle"])

            children = root.iter()
            for child in children:
                if child.tag == "entry":
                    _id = child.attrib["id"]
                    mdt = int(child.attrib["mdt"]) if "mdt" in child.attrib and child.attrib["mdt"] != "" else 0
                    cdt = int(child.attrib["cdt"]) if "cdt" in child.attrib and child.attrib["cdt"] != "" else mdt
                    usr = child.attrib["usr"] if "usr" in child.attrib else ""
                    texts = [c.text for c in child.iter() if c.tag != "src" and c.text is not None and c.text.strip() != ""]

                    signwriting_texts = []
                    spoken_texts = []
                    for text in texts:
                        if is_signwriting(text):
                            signwriting_texts.append(text)
                        else:
                            spoken_texts.append(text)

                    # print(child.attrib)
                    if len(texts) > 0:
                        sample_id = "_".join([str(puddle), _id, str(i)])
                        assumed_spoken_language_code, country_code, sign_language_code = PUDDLES[puddle] if puddle in PUDDLES else ["", "", ""]
                        i += 1
                        yield sample_id, {
                            "puddle": puddle,
                            "id": _id,
                            "assumed_spoken_language_code": assumed_spoken_language_code,
                            "sign_language_code": sign_language_code,
                            "country_code": country_code,
                            "created_date": str(datetime.fromtimestamp(cdt)),
                            "modified_date": str(datetime.fromtimestamp(mdt)),
                            "sign_writing": signwriting_texts,
                            "terms": spoken_texts,
                            "user": usr,
                        }
